/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 25.02.2004
 */
package at.spardat.properties;


/**
 * A property processor is used to process each read property key and value. 
 * Property processors may be chained.
 * Each property processor may change the key or value or stop further processing
 * for the given property.
 *  
 * @author Martin Kahr
 */
public abstract class AbstractPropertyProcessor {
	private AbstractPropertyProcessor next;
	
	protected AbstractPropertyProcessor() {
	}
	
	protected AbstractPropertyProcessor(AbstractPropertyProcessor next) {
		setNextProcessor(next);
	}
	
    /**
     * sets a processor which will be called if this processor successfully processed
     * a property
     * @param next the next processor (in the chain)
     */
	public void setNextProcessor(AbstractPropertyProcessor next) {
		this.next = next;
	}
	
    /**
     * process a property with this property processor
     * @param data the property data object for access to other properties for evaluation
     * @param property the property which shall be processed
     * @return true if the process (in this module and any chained module)
     *         was successfull. False if the process failed.
     * @throws PropertyException
     * @since version_number
     * @author s2867
     */
	public boolean processProperty(PropertyData data, Property property) 
		throws PropertyException
	{
		if (doProcessProperty(data, property)==false) {
			return false;
		} else if (next != null) {
			return next.processProperty(data, property);
		}
		return true;
	}

    /**
     * process a property with this property processor
     * @param data the property data object for access to other properties for evaluation
     * @param property the property which shall be processed
     * @return true if the process in this module was successfull. False if the process failed.
     * @throws PropertyException
     * @since version_number
     * @author s2867
     */    
	protected abstract boolean doProcessProperty(PropertyData data, Property property)
		throws PropertyException;
}
