/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 25.02.2004
 */
package at.spardat.properties;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * encapsulates the information about a single property attribute.
 * Each property attribute may have any number of values. For each value
 * the property keys which are assigned to the given attribute value are
 * stored. 
 * 
 * @author Martin Kahr
 */
public class PropertyAttribute {
	private String name;
	private Map    values;
	
	protected PropertyAttribute(String name) {
		this.name 	= name;
		this.values = new HashMap();
	}
	
	public String getName() { return name; }
	
	public Iterator getPropertiesWithAttributeValue(String attrValue) {
		List properties = (List) values.get(attrValue);
		if (properties == null) return null;
		return properties.iterator();
	}
	public void addPropertyKey(String propertyKey, String[]attrValues) {
		for(int i=0; i < attrValues.length; i++) {
			addPropertyKey(propertyKey, attrValues[i]);
		}
	}
	public void addPropertyKey(String propertyKey, String attrValue) {
		List properties = (List)values.get(attrValue);
		if (properties == null) {
			properties = new ArrayList();
			values.put(attrValue, properties); 
		}
		properties.add(propertyKey);
	}
}
