/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 25.02.2004
 */
package at.spardat.properties;

import java.io.IOException;
import java.io.InputStream;

/**
 * This class is responsible for loading the property files from the
 * filesystem and the classloader into a PropertyData instance.
 * 
 * @author s2867
 */
public class PropertyLoader {
	public static final String ROOT_PROPERTY_FILE_NAME   = "root.properties";
	public static final String SYS_ENV_NAME_FOR_ROOT_FILE= "at.spardat.properties.ROOT_FILE_NAME";
	
	
	/**
	 * Tries to find and load the root property file "root.properties".  
	 * @return PropertyData object which contains all loaded properties 
	 * @throws PropertyInitException in case of syntax errors 
	 */	
	public static PropertyData loadRoot() {
		return loadRoot(PropertyLoader.class.getClassLoader());
	}
	
	/**
	 * Tries to find and load the root property file "root.properties". 
	 * @param loader ClassLoader which shall be used to find the root.properties file 
	 * @return PropertyData object which contains all loaded properties
	 * @throws PropertyInitException in case of syntax errors 
	 */		
	public static PropertyData loadRoot(ClassLoader loader) {
		String overwrittenRootFileName = System.getProperty(SYS_ENV_NAME_FOR_ROOT_FILE);
		if (overwrittenRootFileName!=null)
			return loadRoot(loader, overwrittenRootFileName);
		else
			return loadRoot(loader, ROOT_PROPERTY_FILE_NAME);
			
	}
	
	/**
	 * Tries to find and load the root property file "root.properties". 
	 * @param loader ClassLoader which shall be used to find the root.properties file
	 * @param rootFileName name of the root file 
	 * @return PropertyData object which contains all loaded properties
	 * @throws PropertyInitException in case of syntax errors 
	 */			
	public static PropertyData loadRoot(ClassLoader loader, String rootFileName) {
		ExtendedProperties props = new ExtendedProperties(loader);
		
		InputStream inStream = null;
		try { 
			inStream = loader.getResourceAsStream(rootFileName);
			if (inStream == null) 
				throw new PropertyFileNotFoundException(rootFileName);
			props.read(inStream);
		} catch(IOException ioEx) {
			throw new PropertyInitException("IOException when loading properties", ioEx);			
		} catch(PropertyException e) {
			throw new PropertyInitException("Error when loading properties", e);
		} finally {
			if (inStream != null) try { inStream.close(); } catch(Exception e) {}
		}
		return props.getData();
	}

}
