/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 25.02.2004
 */
package at.spardat.properties.processor;

import java.util.StringTokenizer;

import at.spardat.properties.*;
import at.spardat.properties.Property;
import at.spardat.properties.PropertyData;
import at.spardat.properties.PropertyException;
import at.spardat.properties.PropertySyntaxException;

/**
 * processes conditions of the form 
 * "@cond_key1=cond_val1,cond_val2&cond_key2=cond_val3?key=value"
 * 
 * @author Martin Kahr
 */
public class ConditionProcessor extends AbstractPropertyProcessor {
	public static final String ENVIRONMENT_PROPERTY_NAME = "at.spardat.ENV";

	/**
	 * @see at.spardat.properties.AbstractPropertyProcessor#doProcessProperty(at.spardat.properties.PropertyData, at.spardat.properties.Property)
	 */
	protected boolean doProcessProperty(PropertyData data, Property property) throws PropertyException {
		String key   = property.getKey();
		String value = property.getValue();
		if (key.toString().startsWith("@") && key.toString().startsWith("@include")==false) {
			String line = key + "=" + value;
			// get the correct value and key
			int idx = line.toString().indexOf('?');
			if (idx < 0) {
				throw new PropertySyntaxException(
						"Property '" + key + "=" + value + "' is of incorrect format. '?' is missing.");
			}
			String condition = line.substring(1, idx);
			String prop      = line.substring(idx+1);

			if (evaluateExpression(data, condition) == false) {
				return false;			
			}			
	
			int sepIdx = prop.indexOf('=');
			if (sepIdx < 0) {
				// done by bsen Martin Kaderabek (wenn nach Condition kein = kann es ja ein @include sein, oder
				if(prop.startsWith(IncludeProcessor.INCLUDE_PRAEFIX)) {
					int position = prop.indexOf(' ');
					property.setKey(prop.substring(0, position));
					property.setValue(prop.substring(position + 1));

					return true;
				} else {
					throw new PropertySyntaxException(
							"Property '" + key + "=" + value + "' is of incorrect format. '=' is missing in second part.");
				}
			}
			property.setKey(prop.substring(0, sepIdx));
			property.setValue(prop.substring(sepIdx+1));
		}

		return true;
	}


	/**
	 * evaluates expressions of the form "key=val1,val2&key2=val3,val4"
	 * 
     * @param data the property data object for access to other properties for evaluation
	 * @param expression the expression to evaluate
	 * @return boolean result of expression 
	 */
	private boolean evaluateExpression(PropertyData data, String expression) {
		StringTokenizer token = new StringTokenizer(expression, "&");
		while(token.hasMoreTokens()) {
			if (evaluateSingleExpression(data, token.nextToken())==false)
				return false;
		}
		return true;
	}
	
	/**
	 * evaluates a single expression of the form "key=val1,val2"
	 * 
     * @param data the property data object for access to other properties for evaluation
	 * @param expression the expression to evaluate
	 * @return boolean result of expression 
	 */	
	private boolean evaluateSingleExpression(PropertyData data, String expression) {
		int keyPos   = expression.indexOf('=');
		if (keyPos < 0) {
			// evaluate old environment check of form "@PROD?key=val"
			if (expression.equals(data.getPropertyOrSystemProperty(ENVIRONMENT_PROPERTY_NAME)))
				return true;
			else
				return false;
		}
		
		String key   = expression.substring(0, keyPos);
		String value = expression.substring(keyPos+1);
		StringTokenizer token = new StringTokenizer(value, ",");
		while(token.hasMoreTokens()) {
			String val = token.nextToken();
			if (val.equals(data.getPropertyOrSystemProperty(key)))
				return true;
		}
		
		return false;
	}

}
