/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 25.02.2004
 */
package at.spardat.properties.processor;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import at.spardat.properties.*;
import at.spardat.properties.Property;
import at.spardat.properties.PropertyAttribute;
import at.spardat.properties.PropertyData;
import at.spardat.properties.PropertyException;
import at.spardat.properties.PropertySyntaxException;

/**
 * processes meta data (property attributes)
 * 
 * @author Martin Kahr
 */
public class MetaProcessor extends AbstractPropertyProcessor {
	private PropertyAttribute currentAttribute;
	private String[]		  currentAttributeValues;

	/**
	 * @see at.spardat.properties.AbstractPropertyProcessor#doProcessProperty(at.spardat.properties.PropertyData, at.spardat.properties.Property)
	 */
	protected boolean doProcessProperty(PropertyData data, Property property) throws PropertyException {
		if (property.getKey().startsWith("@meta")) {
			processAttribute(data, property.getValue());
			return false;
		}
		if (currentAttributeValues != null && currentAttribute != null) {
			// TODO: This should only be done if the property is valid - conditions!
			this.currentAttribute.addPropertyKey(property.getKey(), currentAttributeValues);			
		}
		return true;
	}

	public void processAttribute(PropertyData data, String metaTagValue) throws PropertyException {
		int idx = metaTagValue.indexOf('=');		
		if (idx < 0) {
			if (currentAttribute != null && currentAttribute.getName().equals(metaTagValue)==true) {
				currentAttribute = null;
				currentAttributeValues = null;
				return; // group end
			}		
			throw new PropertySyntaxException("The @meta value '" + metaTagValue + "' does not contain the required seperator '='");
		}
		String attrName   = metaTagValue.substring(0, idx);
		String attrValues = metaTagValue.substring(idx+1);
		
		currentAttribute      = data.getAttributes().getOrCreateAttribute(attrName);
		StringTokenizer token = new StringTokenizer(attrValues, ",");
		List values = new ArrayList(); 
		while(token.hasMoreTokens()) {
			String value = token.nextToken();
			values.add(value);
			this.currentAttributeValues = (String[])values.toArray(new String[values.size()]);
		}
		return;
	}
}
