/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 25.02.2004
 */
package at.spardat.properties.processor;

import at.spardat.properties.*;
import at.spardat.properties.Property;
import at.spardat.properties.PropertyData;
import at.spardat.properties.PropertyException;
import at.spardat.properties.PropertyUnknownException;

/**
 * a property processor which manages variable replacement of the form
 * "${varname}" 
 * 
 * @author Martin Kahr
 */
public class VariableProcessor extends AbstractPropertyProcessor {

	protected boolean doProcessProperty(PropertyData data, Property property) 
		throws PropertyException
	{		
		property.setKey(replaceVars(data, property.getKey()));
		property.setValue(replaceVars(data, property.getValue()));
		return true;
	}
	
	protected String replaceVars(PropertyData data, String value)
		throws PropertyUnknownException
	{
		int start = value.indexOf("${");
		int end   = value.indexOf("}", start+2);
		if (start<0 || end < 0) return value;

		String sStart = value.substring(0, start);
		String sEnd   = value.substring(end+1);
		String sVar   = value.substring(start+2, end);
		
		String varVal = data.getPropertyOrSystemProperty(sVar);
		if (varVal == null) {
			throw new PropertyUnknownException(
				"Unable to resolve key '" + sVar + "' for variable replacement" +
				" in '" + value + "'", sVar);
		}
						
		return replaceVars(data, sStart + varVal + sEnd);
	}
	
}
